@@ -295,23 +295,27 @@ static const char* segynames[91] = {
 };
 
 static int help() {
-    puts( "Usage: segyio-catr [OPTION]... FILE\n"
-          "Print specific trace headers from FILE\n"
-          "\n"
-          "-t,  --trace=NUMBER          trace to print\n"
-          "-r,  --range START STOP STEP range of traces to print\n"
-          "-s,  --strict                fail on unreadable tracefields\n"
-          "-S,  --non-strict            don't fail on unreadable tracefields\n"
-          "                             this is the default behaviour\n"
-          "-n,  --segyio-names          print with segyio tracefield names\n"
-          "-v,  --verbose               increase verbosity\n"
-          "     --version               output version information and exit\n"
-          "     --help                  display this help and exit\n"
-          "\n"
+    puts(
+"Usage: segyio-catr [OPTION]... FILE\n"
+"Print specific trace headers from FILE\n"
+"\n"
+"-t,  --trace=NUMBER          trace to print\n"
+"-r,  --range START STOP STEP range of traces to print\n"
+"-f,  --format=FORMAT         override sample format. defaults to inferring\n"
+"                             from the binary header.\n"
+"                             formats: ibm ieee short long char\n"
+"-s,  --strict                fail on unreadable tracefields\n"
+"-S,  --non-strict            don't fail on unreadable tracefields\n"
+"                             this is the default behaviour\n"
+"-k,  --segyio                print with segyio tracefield names\n"
+"-v,  --verbose               increase verbosity\n"
+"     --version               output version information and exit\n"
+"     --help                  display this help and exit\n"
+"\n"
 
-          "the -r flag can takes up to three values: start, stop, step\n"
-          "where all values are defaulted to zero\n"
-          "flags -r and -t can be called multiple times\n"
+"the -r flag takes up to three values: start, stop, step\n"
+"where all values are defaulted to zero\n"
+"flags -r and -t can be used multiple times\n"
         );
     return 0;
 }
@@ -322,6 +326,7 @@ struct options {
     char* src;
     range* r;
     int rsize;
+    int format;
     int verbosity;
     int version, help;
     int strict, labels;
@@ -345,6 +350,7 @@ static struct options parse_options( int argc, char** argv ){
 
     struct options opts;
     opts.rsize = 0; opts.r = calloc( sizeof( range ), rallocsize );
+    opts.format = 0;
     opts.verbosity = 0;
     opts.version = 0; opts.help = 0;
     opts.strict = 0; opts.labels = su_labels;
@@ -353,6 +359,7 @@ static struct options parse_options( int argc, char** argv ){
     static struct option long_options[] = {
         { "trace",          required_argument,  0, 't' },
         { "range",          required_argument,  0, 'r' },
+        { "format",         required_argument,  0, 'f' },
         { "segyio",         no_argument,        0, 'k' },
         { "strict",         no_argument,        0, 's' },
         { "non-strict",     no_argument,        0, 'S' },
@@ -368,7 +375,7 @@ static struct options parse_options( int argc, char** argv ){
     opterr = 1;
     while( true ) {
        int option_index = 0;
-       int c = getopt_long( argc, argv, "sSkvt:r:",
+       int c = getopt_long( argc, argv, "sSkvt:r:f:",
                             long_options, &option_index );
 
        if( c == -1 ) break;
@@ -382,6 +389,23 @@ static struct options parse_options( int argc, char** argv ){
            case 's': opts.strict = 1; break;
            case 'S': opts.strict = 0; break;
            case 'k': opts.labels = segyio_labels; break;
+           case 'f': if( strcmp( optarg, "ibm" ) == 0 )
+                         opts.format = SEGY_IBM_FLOAT_4_BYTE;
+                     if( strcmp( optarg, "ieee" ) == 0 )
+                         opts.format = SEGY_IEEE_FLOAT_4_BYTE;
+                     if( strcmp( optarg, "short" ) == 0 )
+                         opts.format = SEGY_SIGNED_SHORT_2_BYTE;
+                     if( strcmp( optarg, "long" ) == 0 )
+                         opts.format = SEGY_SIGNED_INTEGER_4_BYTE;
+                     if( strcmp( optarg, "char" ) == 0 )
+                         opts.format = SEGY_SIGNED_CHAR_1_BYTE;
+                     if( opts.format == 0 ) {
+                         opts.errmsg = "invalid format argument. valid formats: "
+                                       "ibm ieee short long char";
+                         return opts;
+                     }
+                     break;
+
            case 'r': // intentional fallthrough
            case 't':
                 if( opts.rsize == rallocsize ) {
@@ -474,6 +498,7 @@ int main( int argc, char** argv ) {
         default: labels = su; break;
     }
 
+    /* verify all ranges are sane */
     for( range* r = opts.r; r < opts.r + opts.rsize; ++r ) {
         if( r->start > r->stop && strict )
             exit( errmsg( -3, "Range is empty" ) );
@@ -490,7 +515,36 @@ int main( int argc, char** argv ) {
     if( err ) exit( errmsg( errno, "Unable to read binheader" ) );
 
     int samnr = segy_samples( binheader );
-    int trace_bsize = segy_trace_bsize( samnr );
+
+    int format = opts.format ? opts.format : segy_format( binheader );
+    switch( format ) {
+        /* all good */
+        case SEGY_IBM_FLOAT_4_BYTE:
+        case SEGY_SIGNED_INTEGER_4_BYTE:
+        case SEGY_SIGNED_SHORT_2_BYTE:
+        case SEGY_FIXED_POINT_WITH_GAIN_4_BYTE:
+        case SEGY_IEEE_FLOAT_4_BYTE:
+        case SEGY_SIGNED_CHAR_1_BYTE:
+            break;
+
+        /*
+         * assume this header field is just not set, silently fall back
+         * to 4-byte floats
+         */
+        case 0:
+            format = SEGY_IBM_FLOAT_4_BYTE;
+            break;
+
+        case SEGY_NOT_IN_USE_1:
+        case SEGY_NOT_IN_USE_2:
+        default:
+            errmsg( 1, "sample format field is garbage. "
+                        "falling back to 4-byte float. "
+                        "override with --format" );
+            format = SEGY_IBM_FLOAT_4_BYTE;
+    }
+
+    int trace_bsize = segy_trsize( format, samnr );
     long trace0 = segy_trace0( binheader );
 
     int numtrh;
@@ -498,7 +552,12 @@ int main( int argc, char** argv ) {
     if( err )
         exit( errmsg( errno, "Unable to determine number of traces in file" ) );
 
+    /*
+     * If any range field is defaulted (= 0), expand the defaults into sane
+     * ranges
+     */
     for( range* r = opts.r; r < opts.r + opts.rsize; ++r ) {
+        if( r->start == 0 ) r->start = 1;
         if( r->stop == 0 ) r->stop = r->start;
         if( r->step == 0 ) r->step = 1;
     }
