@@ -331,6 +331,22 @@ static int file_size( FILE* fp, long long* size ) {
 }
 #endif //HAVE_SYS_STAT_H
 
+static int formatsize( int format ) {
+    switch( format ) {
+        case SEGY_IBM_FLOAT_4_BYTE:             return 4;
+        case SEGY_SIGNED_INTEGER_4_BYTE:        return 4;
+        case SEGY_SIGNED_SHORT_2_BYTE:          return 2;
+        case SEGY_FIXED_POINT_WITH_GAIN_4_BYTE: return 4;
+        case SEGY_IEEE_FLOAT_4_BYTE:            return 4;
+        case SEGY_SIGNED_CHAR_1_BYTE:           return 1;
+
+        case SEGY_NOT_IN_USE_1:
+        case SEGY_NOT_IN_USE_2:
+        default:
+            return -1;
+    }
+}
+
 /*
  * addr is NULL if mmap is not found under compilation or if the file is
  * not requested mmap'd. If so, the fallback code path of FILE* is taken
@@ -344,6 +360,7 @@ struct segy_file_handle {
     size_t fsize;
     char mode[ MODEBUF_SIZE ];
     int writable;
+    int elemsize;
 };
 
 segy_file* segy_open( const char* path, const char* mode ) {
@@ -380,6 +397,10 @@ segy_file* segy_open( const char* path, const char* mode ) {
     bool rw = strstr( file->mode, "+" ) || strstr( file->mode, "w" );
     if( rw ) file->writable = 1;
 
+    // assume a size of 4-bytes-per-element, until the set_format function
+    // tells us otherwise.
+    file->elemsize = 4;
+
     return file;
 }
 
@@ -703,6 +724,13 @@ int segy_format( const char* binheader ) {
     return format;
 }
 
+int segy_set_format( segy_file* fp, int format ) {
+    const int elemsize = formatsize( format );
+    if( elemsize < 0 ) return SEGY_INVALID_ARGS;
+    fp->elemsize = elemsize;
+    return SEGY_OK;
+}
+
 int segy_samples( const char* binheader ) {
     int32_t samples = 0;
     segy_get_bfield( binheader, SEGY_BIN_SAMPLES, &samples );
@@ -711,8 +739,13 @@ int segy_samples( const char* binheader ) {
 
 int segy_trace_bsize( int samples ) {
     assert( samples >= 0 );
-    /* Hard four-byte float assumption */
-    return samples * 4;
+    return segy_trsize( SEGY_IBM_FLOAT_4_BYTE, samples );
+}
+
+int segy_trsize( int format, int samples ) {
+    const int elemsize = formatsize( format );
+    if( elemsize < 0 ) return -1;
+    return samples * elemsize;
 }
 
 long segy_trace0( const char* binheader ) {
@@ -855,16 +888,16 @@ int segy_sample_interval( segy_file* fp, float fallback, float* dt ) {
     }
 
     const long trace0 = segy_trace0( bin_header );
-    int samples = segy_samples( bin_header );
-    const int trace_bsize = segy_trace_bsize( samples );
 
-    err = segy_traceheader(fp, 0, trace_header, trace0, trace_bsize);
+    /* we don't need to figure out a trace size, since we're not advancing
+     * beyond the first header */
+    err = segy_traceheader(fp, 0, trace_header, trace0, 0);
     if (err != 0) {
         return err;
     }
 
-    int bindt = 0;
-    int trdt = 0;
+    int32_t bindt = 0;
+    int32_t trdt = 0;
 
     segy_get_bfield( bin_header, SEGY_BIN_INTERVAL, &bindt );
     segy_get_field( trace_header, SEGY_TR_SAMPLE_INTER, &trdt );
@@ -1257,29 +1290,31 @@ static inline int subtr_seek( segy_file* fp,
                               int traceno,
                               int start,
                               int stop,
+                              int elemsize,
                               long trace0,
                               int trace_bsize ) {
     /*
      * Optimistically assume that indices are correct by the time they're given
      * to subtr_seek.
      */
-    int min = start < stop ? start : stop + 1;
-    assert( sizeof( float ) == 4 );
+    const int min = start < stop ? start : stop + 1;
     assert( start >= 0 );
     assert( stop >= -1 );
-    assert( abs(stop - start) * (int)sizeof( float ) <= trace_bsize );
+    assert( abs(stop - start) * elemsize <= trace_bsize );
 
     // skip the trace header and skip everything before min
-    trace0 += (min * (int)sizeof( float )) + SEGY_TRACE_HEADER_SIZE;
+    trace0 += (min * elemsize) + SEGY_TRACE_HEADER_SIZE;
     return segy_seek( fp, traceno, trace0, trace_bsize );
 }
 
-static int reverse( float* arr, int elems ) {
+static int reverse( void* buf, int elems, int elemsize ) {
+    char* arr = (char*) buf;
     const int last = elems - 1;
+    char tmp[ 8 ];
     for( int i = 0; i < elems / 2; ++i ) {
-        const float tmp = arr[ i ];
-        arr[ i ] = arr[ last - i ];
-        arr[ last - i ] = tmp;
+        memcpy( tmp, arr + i * elemsize, elemsize );
+        memcpy( arr + i * elemsize, arr + (last - i) * elemsize, elemsize );
+        memcpy( arr + (last - i) * elemsize, tmp, elemsize );
     }
 
     return SEGY_OK;
@@ -1287,10 +1322,10 @@ static int reverse( float* arr, int elems ) {
 
 int segy_readtrace( segy_file* fp,
                     int traceno,
-                    float* buf,
+                    void* buf,
                     long trace0,
                     int trace_bsize ) {
-    const int stop = trace_bsize / sizeof( float );
+    const int stop = trace_bsize / fp->elemsize;
     return segy_readsubtr( fp, traceno, 0, stop, 1, buf, NULL, trace0, trace_bsize );
 }
 
@@ -1299,28 +1334,29 @@ int segy_readsubtr( segy_file* fp,
                     int start,
                     int stop,
                     int step,
-                    float* buf,
-                    float* rangebuf,
+                    void* buf,
+                    void* rangebuf,
                     long trace0,
                     int trace_bsize ) {
 
-    int err = subtr_seek( fp, traceno, start, stop, trace0, trace_bsize );
-    if( err != SEGY_OK ) return err;
+    const int elems = abs( stop - start );
+    const int elemsize = fp->elemsize;
 
-    const size_t elems = abs( stop - start );
+    int err = subtr_seek( fp, traceno, start, stop, elemsize, trace0, trace_bsize );
+    if( err != SEGY_OK ) return err;
 
     // most common case: step == abs(1), reading contiguously
     if( step == 1 || step == -1 ) {
 
         if( fp->addr ) {
-            err = memread( buf, fp, fp->cur, sizeof( float ) * elems );
+            err = memread( buf, fp, fp->cur, elemsize * elems );
             if( err != SEGY_OK ) return err;
         } else {
-            const size_t readc = fread( buf, sizeof( float ), elems, fp->fp );
+            const int readc = fread( buf, elemsize, elems, fp->fp );
             if( readc != elems ) return SEGY_FREAD_ERROR;
         }
 
-        if( step == -1 ) reverse( buf, elems );
+        if( step == -1 ) reverse( buf, elems, elemsize );
 
         return SEGY_OK;
     }
@@ -1329,10 +1365,15 @@ int segy_readsubtr( segy_file* fp,
     int defstart = start < stop ? 0 : elems - 1;
     int slicelen = slicelength( start, stop, step );
 
+    // step is the distance between elems to read, but now we're counting bytes
+    step *= elemsize;
+    char* dst = (char*)buf;
+
+
     if( fp->addr ) {
-        float* cur = (float*)fp->cur + defstart;
-        for( ; slicelen > 0; cur += step, ++buf, --slicelen )
-            *buf = *cur;
+        const char* cur = (char*)fp->cur + elemsize * defstart;
+        for( ; slicelen > 0; cur += step, dst += elemsize, --slicelen )
+            memcpy( dst, cur, elemsize );
 
         return SEGY_OK;
     }
@@ -1346,17 +1387,17 @@ int segy_readsubtr( segy_file* fp,
      * use, but with a significant performance penalty when no buffer is
      * supplied.
      */
-    float* tracebuf = rangebuf ? rangebuf : malloc( elems * sizeof( float ) );
+    void* tracebuf = rangebuf ? rangebuf : malloc( elems * elemsize );
 
-    const size_t readc = fread( tracebuf, sizeof( float ), elems, fp->fp );
+    const int readc = fread( tracebuf, elemsize, elems, fp->fp );
     if( readc != elems ) {
         if( !rangebuf ) free( tracebuf );
         return SEGY_FREAD_ERROR;
     }
 
-    float* cur = tracebuf + defstart;
-    for( ; slicelen > 0; cur += step, --slicelen, ++buf )
-        *buf = *cur;
+    const char* cur = (char*)tracebuf + elemsize * defstart;
+    for( ; slicelen > 0; cur += step, --slicelen, dst += elemsize )
+        memcpy( dst, cur, elemsize );
 
     if( !rangebuf ) free( tracebuf );
     return SEGY_OK;
@@ -1364,11 +1405,11 @@ int segy_readsubtr( segy_file* fp,
 
 int segy_writetrace( segy_file* fp,
                      int traceno,
-                     const float* buf,
+                     const void* buf,
                      long trace0,
                      int trace_bsize ) {
 
-    const int stop = trace_bsize / sizeof( float );
+    const int stop = trace_bsize / fp->elemsize;
     return segy_writesubtr( fp, traceno, 0, stop, 1, buf, NULL, trace0, trace_bsize );
 }
 
@@ -1377,17 +1418,19 @@ int segy_writesubtr( segy_file* fp,
                      int start,
                      int stop,
                      int step,
-                     const float* buf,
-                     float* rangebuf,
+                     const void* buf,
+                     void* rangebuf,
                      long trace0,
                      int trace_bsize ) {
 
     if( !fp->writable ) return SEGY_READONLY;
 
-    int err = subtr_seek( fp, traceno, start, stop, trace0, trace_bsize );
+    const int elems = abs( stop - start );
+    const int elemsize = fp->elemsize;
+
+    int err = subtr_seek( fp, traceno, start, stop, elemsize, trace0, trace_bsize );
     if( err != SEGY_OK ) return err;
 
-    const size_t elems = abs( stop - start );
 
     if( step == 1 ) {
         /*
@@ -1397,10 +1440,10 @@ int segy_writesubtr( segy_file* fp,
          * be handled by the stride-aware code path
          */
         if( fp->addr ) {
-            err = memread( fp, fp->cur, buf, sizeof( float ) * elems );
+            err = memread( fp, fp->cur, buf, elemsize * elems );
             if( err != SEGY_OK ) return err;
         } else {
-            const size_t writec = fwrite( buf, sizeof( float ), elems, fp->fp );
+            const int writec = fwrite( buf, elemsize, elems, fp->fp );
             if( writec != elems ) return SEGY_FWRITE_ERROR;
         }
 
@@ -1411,33 +1454,36 @@ int segy_writesubtr( segy_file* fp,
     int defstart = start < stop ? 0 : elems - 1;
     int slicelen = slicelength( start, stop, step );
 
+    // step is the distance between elems, but we're counting bytes
+    step *= elemsize;
+    const char* src = (const char*)buf;
+
     if( fp->addr ) {
         /* if mmap is on, strided write is trivial and fast */
-        float* cur = (float*)fp->cur + defstart;
-        for( ; slicelen > 0; cur += step, ++buf, --slicelen )
-            *cur = *buf;
+        char* cur = (char*)fp->cur + elemsize * defstart;
+        for( ; slicelen > 0; cur += step, src += elemsize, --slicelen )
+            memcpy( cur, src, elemsize );
 
         return SEGY_OK;
     }
 
-    const int elemsize = elems * sizeof( float );
-    float* tracebuf = rangebuf ? rangebuf : malloc( elemsize );
+    void* tracebuf = rangebuf ? rangebuf : malloc( elems * elemsize );
 
     // like in readsubtr, read a larger chunk and then step through that
-    const size_t readc = fread( tracebuf, sizeof( float ), elems, fp->fp );
+    const int readc = fread( tracebuf, elemsize, elems, fp->fp );
     if( readc != elems ) { free( tracebuf ); return SEGY_FREAD_ERROR; }
     /* rewind, because fread advances the file pointer */
-    err = fseek( fp->fp, -elemsize, SEEK_CUR );
+    err = fseek( fp->fp, -(elems * elemsize), SEEK_CUR );
     if( err != 0 ) {
         if( !rangebuf ) free( tracebuf );
         return SEGY_FSEEK_ERROR;
     }
 
-    float* cur = tracebuf + defstart;
-    for( ; slicelen > 0; cur += step, --slicelen, ++buf )
-        *cur = *buf;
+    char* cur = (char*)tracebuf + elemsize * defstart;
+    for( ; slicelen > 0; cur += step, --slicelen, src += elemsize )
+        memcpy( cur, src, elemsize );
 
-    const size_t writec = fwrite( tracebuf, sizeof( float ), elems, fp->fp );
+    const int writec = fwrite( tracebuf, elemsize, elems, fp->fp );
     if( !rangebuf ) free( tracebuf );
 
     if( writec != elems ) return SEGY_FWRITE_ERROR;
@@ -1447,20 +1493,35 @@ int segy_writesubtr( segy_file* fp,
 
 int segy_to_native( int format,
                     long long size,
-                    float* buf ) {
+                    void* buf ) {
 
-    assert( sizeof( float ) == sizeof( uint32_t ) );
+    const int elemsize = formatsize( format );
+    if( elemsize < 0 ) return SEGY_INVALID_ARGS;
+    char* dst = (char*)buf;
 
-    uint32_t u;
-    for( long long i = 0; i < size; ++i ) {
-        memcpy( &u, buf + i, sizeof( uint32_t ) );
-        u = ntohl( u );
-        memcpy( buf + i, &u, sizeof( uint32_t ) );
+    if( elemsize == sizeof( uint32_t ) ) {
+        uint32_t u;
+        for( long long i = 0; i < size; ++i ) {
+            const long long pos = i * elemsize;
+            memcpy( &u, dst + pos, sizeof( uint32_t ) );
+            u = ntohl( u );
+            memcpy( dst + pos, &u, sizeof( uint32_t ) );
+        }
+    }
+
+    if( elemsize == sizeof( uint16_t ) ) {
+        uint16_t u;
+        for( long long i = 0; i < size; ++i ) {
+            const long long pos = i * elemsize;
+            memcpy( &u, dst + pos, sizeof( uint16_t ) );
+            u = ntohs( u );
+            memcpy( dst + pos, &u, sizeof( uint16_t ) );
+        }
     }
 
     if( format == SEGY_IBM_FLOAT_4_BYTE ) {
         for( long long i = 0; i < size; ++i )
-            ibm_native( buf + i );
+            ibm_native( dst + i * elemsize );
     }
 
     return SEGY_OK;
@@ -1468,21 +1529,36 @@ int segy_to_native( int format,
 
 int segy_from_native( int format,
                       long long size,
-                      float* buf ) {
+                      void* buf ) {
 
-    assert( sizeof( float ) == sizeof( uint32_t ) );
+    const int elemsize = formatsize( format );
+    if( elemsize < 0 ) return SEGY_INVALID_ARGS;
 
-    uint32_t u;
+    char* dst = (char*)buf;
 
     if( format == SEGY_IBM_FLOAT_4_BYTE ) {
         for( long long i = 0; i < size; ++i )
-            native_ibm( buf + i );
+            native_ibm( dst + i * elemsize );
     }
 
-    for( long long i = 0; i < size; ++i ) {
-        memcpy( &u, buf + i, sizeof( uint32_t ) );
-        u = htonl( u );
-        memcpy( buf + i, &u, sizeof( uint32_t ) );
+    if( elemsize == sizeof( uint32_t ) ) {
+        uint32_t u;
+        for( long long i = 0; i < size; ++i ) {
+            const long long pos = i * elemsize;
+            memcpy( &u, dst + pos, sizeof( uint32_t ) );
+            u = htonl( u );
+            memcpy( dst + pos, &u, sizeof( uint32_t ) );
+        }
+    }
+
+    if( elemsize == sizeof( uint16_t ) ) {
+        uint16_t u;
+        for( long long i = 0; i < size; ++i ) {
+            const long long pos = i * elemsize;
+            memcpy( &u, dst + pos, sizeof( uint16_t ) );
+            u = htons( u );
+            memcpy( dst + pos, &u, sizeof( uint16_t ) );
+        }
     }
 
     return SEGY_OK;
@@ -1520,18 +1596,15 @@ int segy_read_line( segy_file* fp,
                     int line_length,
                     int stride,
                     int offsets,
-                    float* buf,
+                    void* buf,
                     long trace0,
                     int trace_bsize ) {
 
-    assert( sizeof( float ) == sizeof( int32_t ) );
-    assert( trace_bsize % 4 == 0 );
-    const int trace_data_size = trace_bsize / 4;
-
+    char* dst = (char*) buf;
     stride *= offsets;
 
-    for( ; line_length--; line_trace0 += stride, buf += trace_data_size ) {
-        int err = segy_readtrace( fp, line_trace0, buf, trace0, trace_bsize );
+    for( ; line_length--; line_trace0 += stride, dst += trace_bsize ) {
+        int err = segy_readtrace( fp, line_trace0, dst, trace0, trace_bsize );
         if( err != 0 ) return err;
     }
 
@@ -1554,20 +1627,16 @@ int segy_write_line( segy_file* fp,
                      int line_length,
                      int stride,
                      int offsets,
-                     const float* buf,
+                     const void* buf,
                      long trace0,
                      int trace_bsize ) {
     if( !fp->writable ) return SEGY_READONLY;
 
-    assert( sizeof( float ) == sizeof( int32_t ) );
-    assert( trace_bsize % 4 == 0 );
-    const int trace_data_size = trace_bsize / 4;
-
-    line_trace0 *= offsets;
+    const char* src = (const char*) buf;
     stride *= offsets;
 
-    for( ; line_length--; line_trace0 += stride, buf += trace_data_size ) {
-        int err = segy_writetrace( fp, line_trace0, buf, trace0, trace_bsize );
+    for( ; line_length--; line_trace0 += stride, src += trace_bsize ) {
+        int err = segy_writetrace( fp, line_trace0, src, trace0, trace_bsize );
         if( err != 0 ) return err;
     }
 
