@@ -11,6 +11,7 @@ import itertools
 import filecmp
 import shutil
 import numpy as np
+import numpy.testing as npt
 import pytest
 from pytest import approx
 
@@ -18,10 +19,9 @@ from test import tmpfiles
 
 import segyio
 from segyio import TraceField, BinField
-from segyio._field import Field
-from segyio._line import Line
-from segyio._header import Header
-from segyio._trace import Trace
+from segyio.field import Field
+from segyio.line import Line, HeaderLine
+from segyio.trace import Trace, Header
 
 
 def test_inline_4():
@@ -31,29 +31,29 @@ def test_inline_4():
 
         data = f.iline[4]
 
-        assert 4.2, approx(data[0, 0], abs=1e-6)
+        assert 4.2 == approx(data[0, 0], abs=1e-6)
         # middle sample
-        assert 4.20024, approx(data[0, sample_count // 2 - 1], abs=1e-6)
+        assert 4.20024 == approx(data[0, sample_count // 2 - 1], abs=1e-6)
         # last sample
-        assert 4.20049, approx(data[0, -1], abs=1e-6)
+        assert 4.20049 == approx(data[0, -1], abs=1e-6)
 
         # middle xline
         middle_line = 2
         # first sample
-        assert 4.22, approx(data[middle_line, 0], abs=1e-5)
+        assert 4.22 == approx(data[middle_line, 0], abs=1e-5)
         # middle sample
-        assert 4.22024, approx(data[middle_line, sample_count // 2 - 1], abs=1e-6)
+        assert 4.22024 == approx(data[middle_line, sample_count // 2 - 1], abs=1e-6)
         # last sample
-        assert 4.22049, approx(data[middle_line, -1], abs=1e-6)
+        assert 4.22049 == approx(data[middle_line, -1], abs=1e-6)
 
         # last xline
         last_line = (len(f.xlines) - 1)
         # first sample
-        assert 4.24, approx(data[last_line, 0], abs=1e-5)
+        assert 4.24 == approx(data[last_line, 0], abs=1e-5)
         # middle sample
-        assert 4.24024, approx(data[last_line, sample_count // 2 - 1], abs=1e-6)
+        assert 4.24024 == approx(data[last_line, sample_count // 2 - 1], abs=1e-6)
         # last sample
-        assert 4.24049, approx(data[last_line, sample_count - 1], abs=1e-6)
+        assert 4.24049 == approx(data[last_line, sample_count - 1], abs=1e-6)
 
 
 def test_xline_22():
@@ -64,29 +64,29 @@ def test_xline_22():
 
         # first iline
         # first sample
-        assert 1.22, approx(data[0, 0], abs=1e-5)
+        assert 1.22 == approx(data[0, 0], abs=1e-5)
         # middle sample
-        assert 1.22024, approx(data[0, size // 2 - 1], abs=1e-6)
+        assert 1.22024 == approx(data[0, size // 2 - 1], abs=1e-6)
         # last sample
-        assert 1.22049, approx(data[0, size - 1], abs=1e-6)
+        assert 1.22049 == approx(data[0, size - 1], abs=1e-6)
 
         # middle iline
         middle_line = 2
         # first sample
-        assert 3.22, approx(data[middle_line, 0], abs=1e-5)
+        assert 3.22 == approx(data[middle_line, 0], abs=1e-5)
         # middle sample
-        assert 3.22024, approx(data[middle_line, size // 2 - 1], abs=1e-6)
+        assert 3.22024 == approx(data[middle_line, size // 2 - 1], abs=1e-6)
         # last sample
-        assert 3.22049, approx(data[middle_line, size - 1], abs=1e-6)
+        assert 3.22049 == approx(data[middle_line, size - 1], abs=1e-6)
 
         # last iline
         last_line = len(f.ilines) - 1
         # first sample
-        assert 5.22, approx(data[last_line, 0], abs=1e-5)
+        assert 5.22 == approx(data[last_line, 0], abs=1e-5)
         # middle sample
-        assert 5.22024, approx(data[last_line, size // 2 - 1], abs=1e-6)
+        assert 5.22024 == approx(data[last_line, size // 2 - 1], abs=1e-6)
         # last sample
-        assert 5.22049, approx(data[last_line, size - 1], abs=1e-6)
+        assert 5.22049 == approx(data[last_line, size - 1], abs=1e-6)
 
 
 def test_iline_slicing():
@@ -130,6 +130,7 @@ def test_file_info():
         assert 2 == f.sorting
         assert 1 == f.offsets
         assert 1 == int(f.format)
+        assert np.single == f.dtype
 
         xlines = list(range(20, 25))
         ilines = list(range(1, 6))
@@ -165,8 +166,7 @@ def test_traces_slicing():
         assert rev_traces[2][49] == f.trace[0][49]
 
         # make sure buffers can be reused
-        buf = None
-        for i, trace in enumerate(f.trace[0:6:2, buf]):
+        for i, trace in enumerate(f.trace[0:6:2]):
             assert np.array_equal(trace, traces[i])
 
 
@@ -214,6 +214,37 @@ def test_header_dict_methods():
         iter(f.bin)
 
 
+@tmpfiles('test-data/small.sgy')
+def test_header_dropped_writes(tmpdir):
+    with segyio.open('test-data/small.sgy', mode='r+') as f:
+        f.header[10] = { 1: 5, 5: 10 }
+
+    with segyio.open('test-data/small.sgy', mode='r+') as f:
+        x, y = f.header[10], f.header[10]
+
+        assert x[1, 5] == { 1: 5, 5: 10 }
+        assert y[1, 5] == { 1: 5, 5: 10 }
+
+        # write to x[1] is invisible to y
+        x[1] = 6
+        assert x[1] == 6
+        assert y[1] == 5
+
+        y.reload()
+        assert x[1] == 6
+        assert y[1] == 6
+
+        x[1] = 5
+        assert x[1] == 5
+        assert y[1] == 6
+
+        # the write to x[1] is lost
+        y[5] = 1
+        assert x[1] == 5
+        assert x.reload()
+        assert x[1] == 6
+        assert y[1, 5] == { 1: 6, 5: 1 }
+
 @tmpfiles("test-data/small-ps.sgy")
 def test_headers_line_offset(tmpdir):
     il, xl = TraceField.INLINE_3D, TraceField.CROSSLINE_3D
@@ -456,13 +487,13 @@ def test_read_header():
         with pytest.raises(IndexError):
             _ = f.header[-30]
 
-        with pytest.raises(IndexError):
+        with pytest.raises(KeyError):
             _ = f.header[0][188]  # between byte offsets
 
-        with pytest.raises(IndexError):
+        with pytest.raises(KeyError):
             _ = f.header[0][-1]
 
-        with pytest.raises(IndexError):
+        with pytest.raises(KeyError):
             _ = f.header[0][700]
 
 
@@ -477,13 +508,13 @@ def test_write_header(tmpdir):
         assert 1 == f.header[1][189]
 
         # accessing non-existing offsets raises exceptions
-        with pytest.raises(IndexError):
+        with pytest.raises(KeyError):
             f.header[0][188] = 1  # between byte offsets
 
-        with pytest.raises(IndexError):
+        with pytest.raises(KeyError):
             f.header[0][-1] = 1
 
-        with pytest.raises(IndexError):
+        with pytest.raises(KeyError):
             f.header[0][700] = 1
 
         d = {TraceField.INLINE_3D: 43,
@@ -523,6 +554,8 @@ def test_write_header(tmpdir):
         # accept anything with a key-value structure
         f.header[5].update([(segyio.su.ns, 12), (segyio.su.dt, 4)])
         f.header[5].update(((segyio.su.muts, 3), (segyio.su.mute, 7)))
+        f.header[5].update([(segyio.su.muts, 3)], sx=7)
+        f.header[5].update(sy=8)
 
         with pytest.raises(TypeError):
             f.header[0].update(10)
@@ -538,6 +571,8 @@ def test_write_header(tmpdir):
         assert 4 == f.header[5][segyio.su.dt]
         assert 3 == f.header[5][segyio.su.muts]
         assert 7 == f.header[5][segyio.su.mute]
+        assert 7 == f.header[5][segyio.su.sx]
+        assert 8 == f.header[5][segyio.su.sy]
 
         # for-each support
         for _ in f.header:
@@ -547,6 +582,19 @@ def test_write_header(tmpdir):
         f.header[2] = f.header[1]
         f.flush()
 
+        d = {TraceField.INLINE_3D: 12,
+             TraceField.CROSSLINE_3D: 13,
+             TraceField.offset: 14}
+
+        # assign multiple values with a slice
+        f.header[:5] = d
+        f.flush()
+
+        for i in range(5):
+            assert 12 == f.header[i][TraceField.INLINE_3D]
+            assert 13 == f.header[i][segyio.su.xline]
+            assert 14 == f.header[i][segyio.su.offset]
+
         # don't use this interface in production code, it's only for testing
         # i.e. don't access buf of treat it as a list
         # assertEqual(list(f.header[2].buf), list(f.header[1].buf))
@@ -561,13 +609,13 @@ def test_write_binary(tmpdir):
         assert 5 == f.bin[3213]
 
         # accessing non-existing offsets raises exceptions
-        with pytest.raises(IndexError):
+        with pytest.raises(KeyError):
             _ = f.bin[0]
 
-        with pytest.raises(IndexError):
+        with pytest.raises(KeyError):
             _ = f.bin[50000]
 
-        with pytest.raises(IndexError):
+        with pytest.raises(KeyError):
             _ = f.bin[3214]
 
         d = {BinField.Traces: 43,
@@ -605,6 +653,43 @@ def test_write_binary(tmpdir):
         # copy a header
         f.bin = f.bin
 
+@tmpfiles("test-data/small.sgy")
+def test_write_header_update_atomic(tmpdir):
+    with segyio.open(tmpdir / "small.sgy", "r+") as f:
+        orig = dict(f.header[10])
+
+        d = { 1:  10,
+              37: 4,
+              73: 15,
+              2:  10, # this key raises error
+            }
+
+        # use the same instance all the time, to also catch the case where
+        # update dirties the backing storage
+        header = f.header[10]
+        with pytest.raises(KeyError):
+            header.update(d)
+
+        assert orig == header
+
+        # flushing the header should just write a clean buffer
+        header.flush()
+        assert orig == header
+
+        del d[2]
+        header.update(d)
+
+        assert header[1] == 10
+        assert header[73] == 15
+        assert f.header[10][37] == 4
+
+        fresh = orig.copy()
+        fresh.update(d)
+
+        assert orig != header
+        assert orig != f.header[10]
+        assert fresh == f.header[10]
+        assert header == f.header[10]
 
 def test_fopen_error():
     # non-existent file
@@ -675,18 +760,62 @@ def test_open_fails_unstructured():
         assert (list(f.attributes(189)[:]) ==
                 [(i // 5) + 1 for i in range(len(f.trace))])
 
+@tmpfiles('test-data/small.sgy')
+def test_write_with_narrowing(tmpdir):
+    with segyio.open(tmpdir / 'small.sgy', mode = 'r+') as f:
+
+        with pytest.warns(RuntimeWarning):
+            ones = np.ones(len(f.samples), dtype=np.float64)
+            f.trace[0] = ones
+            assert np.array_equal(f.trace[0], ones)
+
+        with pytest.warns(RuntimeWarning):
+            twos = [np.single(2.0) for _ in range(len(f.samples))]
+            f.trace[1] = twos
+            assert np.array_equal(f.trace[1], twos)
+
+        with pytest.warns(RuntimeWarning):
+            first = f.ilines[0]
+            shape = f.iline[first].shape
+            ones = np.ones(shape, dtype=np.float64)
+            f.iline[first] = ones
+            assert np.array_equal(f.iline[first], ones)
+
+        with pytest.warns(RuntimeWarning):
+            last = f.xlines[-1]
+            shape = f.xline[last].shape
+            threes = np.ones(shape, dtype=np.float64) * 3
+            f.xline[last] = threes
+            assert np.array_equal(f.xline[last], threes)
+
+@tmpfiles('test-data/small.sgy')
+def test_write_with_array_likes(tmpdir):
+    with segyio.open(tmpdir / 'small.sgy', mode = 'r+') as f:
+
+        with pytest.warns(RuntimeWarning):
+            ones = np.ones(3 * len(f.samples), dtype='single')
+            # ::3 makes the array non-contiguous
+            f.trace[0] = ones[::3]
+            assert np.array_equal(f.trace[0], ones[::3])
+
+        with pytest.warns(RuntimeWarning):
+            ones = np.ones(len(f.samples), dtype='single')
+            f.trace[0] = (1 for _ in range(len(f.samples)))
+            assert np.array_equal(f.trace[0], ones)
 
 @tmpfiles("test-data/small.sgy")
 def test_assign_all_traces(tmpdir):
-    shutil.copy(tmpdir / 'small.sgy', tmpdir / 'copy_small.sgy')
+    orig = str(tmpdir / 'small.sgy')
+    copy = str(tmpdir / 'copy.sgy')
+    shutil.copy(orig, copy)
 
-    with segyio.open(tmpdir / 'small.sgy') as f:
+    with segyio.open(orig) as f:
         traces = f.trace.raw[:] * 2.0
 
-    with segyio.open(tmpdir / 'copy_small.sgy', 'r+') as f:
+    with segyio.open(copy, 'r+') as f:
         f.trace[:] = traces[:]
 
-    with segyio.open(tmpdir / 'copy_small.sgy') as f:
+    with segyio.open(copy) as f:
         assert np.array_equal(f.trace.raw[:], traces)
 
 
@@ -704,7 +833,9 @@ def test_traceaccess_from_array():
 
 @tmpfiles("test-data/small.sgy")
 def test_create_sgy(tmpdir):
-    with segyio.open(tmpdir / "small.sgy") as src:
+    small = str(tmpdir / 'small.sgy')
+    fresh = str(tmpdir / 'fresh.sgy')
+    with segyio.open(small) as src:
         spec = segyio.spec()
         spec.format = int(src.format)
         spec.sorting = int(src.sorting)
@@ -712,7 +843,7 @@ def test_create_sgy(tmpdir):
         spec.ilines = src.ilines
         spec.xlines = src.xlines
 
-        with segyio.create(tmpdir / "small_created.sgy", spec) as dst:
+        with segyio.create(fresh, spec) as dst:
             dst.text[0] = src.text[0]
             dst.bin = src.bin
 
@@ -732,19 +863,170 @@ def test_create_sgy(tmpdir):
             # for dsttr, srctr in zip(dst.trace, src.trace):
             #    dsttr = srctr
 
-    assert filecmp.cmp(tmpdir / "small.sgy", tmpdir / "small_created.sgy")
+    assert filecmp.cmp(small, fresh)
+
+@tmpfiles("test-data/small.sgy")
+def test_ref_getitem(tmpdir):
+    small = str(tmpdir / 'small.sgy')
+    with segyio.open(small, mode = 'r+') as f:
+        with f.trace.ref as ref:
+            expected = ref[10].copy()
+            x = ref[10]
+            y = ref[10]
+            x[5] = 0
+            # getting a new trace within the same with block should not
+            # invalidate other refs
+            y = ref[11]
+            x[6] = 1.6721
+            x[5] = 52
+            y[0] = 0
+            assert ref[10][5] == 52
+            assert ref[10][6] == approx(1.6721)
+
+            assert len(ref.refs) == 2
+            z = ref[12]
+            z[0] = 0
+            assert len(ref.refs) == 3
+
+            x, y = None, None
+
+            ref.flush()
+            assert len(ref.refs) == 1
+
+    with segyio.open(small) as f:
+        expected[5] = 52
+        expected[6] = 1.6721
+        npt.assert_array_almost_equal(expected, f.trace[10])
 
 
+@tmpfiles("test-data/small.sgy")
+def test_ref_inplace_add_foreach(tmpdir):
+    small = str(tmpdir / 'small.sgy')
+    with segyio.open(small, mode = 'r+') as f:
+        expected = f.trace.raw[:] + 1.617
+
+        with f.trace.ref as ref:
+            for x in ref[:]:
+                x += 1.617
+
+    with segyio.open(small) as f:
+        npt.assert_array_almost_equal(expected, f.trace.raw[:])
+
+@tmpfiles("test-data/small.sgy")
+def test_ref_preserve_change_except_block(tmpdir):
+    small = str(tmpdir / 'small.sgy')
+    with segyio.open(small, mode = 'r+') as f:
+        expected = f.trace.raw[:]
+        expected[10][0] = 0
+
+        with f.trace.ref as ref:
+            try:
+                for i, x in enumerate(ref[:]):
+                    if i == 10: raise RuntimeError
+            except RuntimeError:
+                x[0] = 0
+
+    with segyio.open(small) as f:
+        result = f.trace.raw[:]
+        npt.assert_array_almost_equal(expected[10], result[10])
+        npt.assert_array_almost_equal(expected, result)
+
+
+@tmpfiles("test-data/small.sgy")
+def test_ref_post_loop_var(tmpdir):
+    small = str(tmpdir / 'small.sgy')
+    with segyio.open(small, mode = 'r+') as f:
+        expected = f.trace[-1]
+        expected[0] = 1.617
+
+        with f.trace.ref as ref:
+            for x in ref[:]: pass
+
+            x[0] = 1.617
+
+    with segyio.open(small) as f:
+        npt.assert_array_almost_equal(expected, f.trace[-1])
+
+@tmpfiles("test-data/small.sgy")
+def test_ref_sliced(tmpdir):
+    small = str(tmpdir / 'small.sgy')
+    with segyio.open(small, mode = 'r+') as f:
+        expected = f.trace.raw[:]
+        expected[10:15] += expected[:5]
+
+        with f.trace.ref as ref:
+            for x, y in zip(ref[10:15], f.trace[:]):
+                np.copyto(x, x + y)
+
+    with segyio.open(small) as f:
+        npt.assert_array_almost_equal(expected, f.trace.raw[:])
+
+@tmpfiles("test-data/small.sgy")
+def test_ref_mixed_for_else(tmpdir):
+    small = str(tmpdir / 'small.sgy')
+    with segyio.open(small, mode = 'r+') as f:
+        samples = len(f.samples)
+        zeros = np.zeros(samples, dtype = f.dtype)
+        ones = np.ones(samples, dtype = f.dtype)
+        expected = f.trace.raw[:]
+        expected[7] = zeros
+        expected[5:10] += 1.617
+        expected[9] = ones
+
+        with f.trace.ref as ref:
+            ref[7] = zeros
+            for x in ref[5:10]:
+                # break won't run, so enter else block
+                if len(x) == 0: break
+                x += 1.617
+            else: # else should preserve last x
+                np.copyto(x, ones)
+
+    with segyio.open(small) as f:
+        npt.assert_array_almost_equal(expected, f.trace.raw[:])
+
+
+@tmpfiles("test-data/small.sgy")
+def test_ref_new_file(tmpdir):
+    # this is the case the trace.ref feature was designed to support, namely
+    # creating a file trace-by-trace based on some transformation of another
+    # file, or an operation on multiple, where the trace index itself is
+    # uninteresting.
+
+    small = str(tmpdir / 'small.sgy')
+    fresh = str(tmpdir / 'fresh.sgy')
+    with segyio.open(small) as src:
+
+        spec = segyio.tools.metadata(src)
+        with segyio.create(fresh, spec) as dst:
+            dst.text[0] = src.text[0]
+            dst.bin = src.bin
+
+            # copy all headers
+            dst.header = src.header
+
+            with dst.trace.ref as ref:
+                for x, y in zip(ref[:], src.trace):
+                    # x default-inits to 0, so += is essentially copyto()
+                    # since traces hasn't been written yet, this has to handle
+                    # read miss errors
+                    x += y
+
+        with segyio.open(fresh) as dst:
+            npt.assert_array_almost_equal(src.trace.raw[:], dst.trace.raw[:])
+
 @tmpfiles("test-data/small.sgy")
 def test_create_sgy_truncate(tmpdir):
-    with segyio.open(tmpdir / "small.sgy") as src:
+    small = str(tmpdir / 'small.sgy')
+    trunc = str(tmpdir / 'text-truncated.sgy')
+    with segyio.open(small) as src:
         spec = segyio.tools.metadata(src)
 
         # repeat the text header 3 times
         text = src.text[0]
         text = text + text + text
 
-        with segyio.create(tmpdir / "text-truncated.sgy", spec) as dst:
+        with segyio.create(trunc, spec) as dst:
             dst.bin = src.bin
             dst.text[0] = text
 
@@ -754,7 +1036,7 @@ def test_create_sgy_truncate(tmpdir):
 
             dst.trace = src.trace
 
-    assert filecmp.cmp(tmpdir / "small.sgy", tmpdir / "text-truncated.sgy")
+    assert filecmp.cmp(small, trunc)
 
 
 @tmpfiles("test-data/small.sgy")
@@ -786,7 +1068,6 @@ def test_create_sgy_shorter_traces(tmpdir):
             assert 20 == len(dst.samples)
             assert [x + 100 for x in src.ilines] == list(dst.ilines)
 
-
 def test_create_from_naught(tmpdir):
     spec = segyio.spec()
     spec.format = 5
@@ -812,10 +1093,10 @@ def test_create_from_naught(tmpdir):
         dst.header = {TraceField.offset: 1}
 
     with segyio.open(tmpdir / "mk.sgy") as f:
-        assert 1, approx(f.trace[0][0], abs=1e-4)
-        assert 1.001, approx(f.trace[0][1], abs=1e-4)
-        assert 1.149, approx(f.trace[0][-1], abs=1e-4)
-        assert 50.100, approx(f.trace[-1][100], abs=1e-4)
+        assert 1 == approx(f.trace[0][0], abs=1e-4)
+        assert 1.001 == approx(f.trace[0][1], abs=1e-4)
+        assert 1.149 == approx(f.trace[0][-1], abs=1e-4)
+        assert 50.100 == approx(f.trace[-1][100], abs=1e-4)
         assert f.header[0][TraceField.offset] == f.header[1][TraceField.offset]
         assert 1 == f.header[1][TraceField.offset]
 
@@ -850,10 +1131,10 @@ def test_create_from_naught_prestack(tmpdir):
                 dst.header.xline[xl, of] = {TraceField.CROSSLINE_3D: xl}
 
     with segyio.open(tmpdir / "mk-ps.sgy") as f:
-        assert 101.010, approx(f.trace[0][0], abs=1e-4)
-        assert 101.011, approx(f.trace[0][1], abs=1e-4)
-        assert 101.016, approx(f.trace[0][-1], abs=1e-4)
-        assert 503.025, approx(f.trace[-1][5], abs=1e-4)
+        assert 101.010 == approx(f.trace[0][0], abs=1e-4)
+        assert 101.011 == approx(f.trace[0][1], abs=1e-4)
+        assert 101.016 == approx(f.trace[0][-1], abs=1e-4)
+        assert 503.025 == approx(f.trace[-1][5], abs=1e-4)
         assert f.header[0][TraceField.offset] != f.header[1][TraceField.offset]
         assert 1 == f.header[0][TraceField.offset]
         assert 2 == f.header[1][TraceField.offset]
@@ -861,15 +1142,53 @@ def test_create_from_naught_prestack(tmpdir):
         for x, y in zip(f.iline[:, :], cube):
             assert list(x.flatten()) == list(y.flatten())
 
+def test_create_unstructured_hasattrs(tmpdir):
+    spec = segyio.spec()
+    spec.format = 5
+    spec.samples = range(150)
+    spec.tracecount = 50
+
+    with segyio.create(tmpdir / "mk.sgy", spec) as dst:
+        # accessing the sorting, inline and crossline attributes should work,
+        # but not raise errors
+        assert not dst.sorting
+        assert not dst.ilines
+        assert not dst.xlines
+        assert dst.unstructured
+
+def test_create_from_naught_unstructured(tmpdir):
+    spec = segyio.spec()
+    spec.format = 5
+    spec.samples = range(150)
+    spec.tracecount = 50
+
+    with segyio.create(tmpdir / "unstructured.sgy", spec) as dst:
+        tr = np.array(range(150), dtype = np.single)
+
+        for i in range(len(dst.trace)):
+            dst.trace[i] = tr
+            tr += 1
+
+        # Set header field 'offset' to 1 in all headers
+        dst.header = {TraceField.offset: 1}
+
+    with segyio.open(tmpdir / "unstructured.sgy") as f:
+        assert 1 == approx(f.trace[1][0], abs=1e-4)
+        assert 2 == approx(f.trace[1][1], abs=1e-4)
+        assert 150 == approx(f.trace[1][-1], abs=1e-4)
+        assert 149 == approx(f.trace[-1][100], abs=1e-4)
+        assert f.header[10][TraceField.offset] == f.header[25][TraceField.offset]
+        assert 1 == f.header[1][TraceField.offset]
+
 
 def test_create_write_lines(tmpdir):
     mklines(tmpdir / "mklines.sgy")
 
     with segyio.open(tmpdir / "mklines.sgy") as f:
-        assert 1, approx(f.iline[1][0][0], abs=1e-4)
-        assert 2.004, approx(f.iline[2][0][4], abs=1e-4)
-        assert 2.014, approx(f.iline[2][1][4], abs=1e-4)
-        assert 8.043, approx(f.iline[8][4][3], abs=1e-4)
+        assert 1 == approx(f.iline[1][0][0], abs=1e-4)
+        assert 2.004 == approx(f.iline[2][0][4], abs=1e-4)
+        assert 2.014 == approx(f.iline[2][1][4], abs=1e-4)
+        assert 8.043 == approx(f.iline[8][4][3], abs=1e-4)
 
 
 def test_create_sgy_skip_lines(tmpdir):
@@ -932,8 +1251,7 @@ def mklines(fname):
         for xl in spec.xlines:
             dst.header.xline[xl] = {TraceField.CROSSLINE_3D: xl}
 
-
-def test_create_bad_specs():
+def test_create_bad_specs(tmpdir):
     class C:
         pass
 
@@ -948,35 +1266,34 @@ def test_create_bad_specs():
     for attr, val in mandatory:
         setattr(c, attr, val)
         with pytest.raises(AttributeError):
-            with segyio.create("foo", c):
+            with segyio.create(tmpdir / 'foo' + attr, c):
                 pass
 
     c.tracecount = 10
-    with segyio.create("foo", c):
+    with segyio.create(tmpdir / 'tracecount', c):
         pass
 
     del c.tracecount
 
     c.ilines = [1, 2, 3]
     with pytest.raises(AttributeError):
-        with segyio.create("foo", c):
+        with segyio.create(tmpdir / 'ilines', c):
             pass
 
     c.xlines = [4, 6, 8]
     with pytest.raises(AttributeError):
-        with segyio.create("foo", c):
+        with segyio.create(tmpdir / 'xlines', c):
             pass
 
     c.offsets = [1]
     with pytest.raises(AttributeError):
-        with segyio.create("foo", c):
+        with segyio.create(tmpdir / 'offsets', c):
             pass
 
     c.sorting = 2
-    with segyio.create("foo", c):
+    with segyio.create(tmpdir / 'ok.sgy', c):
         pass
 
-
 def test_segyio_types():
     with segyio.open("test-data/small.sgy") as f:
         assert isinstance(f.sorting, int)
@@ -984,7 +1301,8 @@ def test_segyio_types():
         assert isinstance(f.tracecount, int)
         assert isinstance(f.samples, np.ndarray)
 
-        assert isinstance(f.depth_slice, Line)
+        from segyio.depth import Depth
+        assert isinstance(f.depth_slice, Depth)
         assert isinstance(f.depth_slice[1], np.ndarray)
         assert isinstance(f.depth_slice[1:23], GeneratorType)
 
@@ -1007,10 +1325,10 @@ def test_segyio_types():
         assert isinstance(f.xline[21][0][0:3], np.ndarray)
 
         assert isinstance(f.header, Header)
-        assert isinstance(f.header.iline, Line)
+        assert isinstance(f.header.iline, HeaderLine)
         assert isinstance(f.header.iline[1], GeneratorType)
         assert isinstance(next(f.header.iline[1]), Field)
-        assert isinstance(f.header.xline, Line)
+        assert isinstance(f.header.xline, HeaderLine)
         assert isinstance(f.header.xline[21], GeneratorType)
         assert isinstance(next(f.header.xline[21]), Field)
 
@@ -1022,10 +1340,11 @@ def test_segyio_types():
 
 
 def test_depth_slice_reading():
+    from itertools import islice
     with segyio.open("test-data/small.sgy") as f:
         assert len(f.depth_slice) == len(f.samples)
 
-        for depth_sample in range(len(f.samples)):
+        for depth_sample in range(len(f.samples))[::5]:
             depth_slice = f.depth_slice[depth_sample]
             assert isinstance(depth_slice, np.ndarray)
             assert depth_slice.shape == (5, 5)
@@ -1034,7 +1353,8 @@ def test_depth_slice_reading():
                 i, j = x - f.ilines[0], y - f.xlines[0]
                 assert depth_slice[i][j] == approx(f.iline[x][j][depth_sample], abs=1e-6)
 
-        for index, depth_slice in enumerate(f.depth_slice):
+        itr = iter(enumerate(f.depth_slice))
+        for index, depth_slice in itr:
             assert isinstance(depth_slice, np.ndarray)
             assert depth_slice.shape == (5, 5)
 
@@ -1042,12 +1362,15 @@ def test_depth_slice_reading():
                 i, j = x - f.ilines[0], y - f.xlines[0]
                 assert depth_slice[i][j] == approx(f.iline[x][j][index], abs=1e-6)
 
-    with pytest.raises(KeyError):
+            next(islice(itr, 5, 5), None)
+
+    with pytest.raises(IndexError):
         _ = f.depth_slice[len(f.samples)]
 
 
 @tmpfiles("test-data/small.sgy")
 def test_depth_slice_writing(tmpdir):
+    from itertools import islice
 
     buf = np.empty(shape=(5, 5), dtype=np.single)
 
@@ -1062,8 +1385,10 @@ def test_depth_slice_writing(tmpdir):
         assert np.allclose(f.depth_slice[7], buf * 3.14)
 
         f.depth_slice = [buf * i for i in range(len(f.depth_slice))]  # assign to all depths
-        for index, depth_slice in enumerate(f.depth_slice):
+        itr = iter(enumerate(f.depth_slice))
+        for index, depth_slice in itr:
             assert np.allclose(depth_slice, buf * index)
+            next(islice(itr, 3, 3), None)
 
 def test_no_16bit_overflow_tracecount(tmpdir):
     spec = segyio.spec()
@@ -1080,10 +1405,61 @@ def test_no_16bit_overflow_tracecount(tmpdir):
     with segyio.create(tmpdir / 'foo.sgy', spec) as f:
         assert f.tracecount > 0
         assert f.tracecount > 2**16 - 1
-        for i in range(f.tracecount):
-            f.trace[i] = ones
-            f.header[i] = {
-                    segyio.TraceField.INLINE_3D: i,
-                    segyio.TraceField.CROSSLINE_3D: i,
+        f.trace[-1] = ones
+        f.header[-1] = {
+                    segyio.TraceField.INLINE_3D: 10,
+                    segyio.TraceField.CROSSLINE_3D: 10,
                     segyio.TraceField.offset: 1,
-            }
+        }
+
+def test_open_2byte_int_format():
+    with segyio.open('test-data/f3.sgy') as f:
+        assert int(f.format)  == 3
+        assert len(f.samples) == 75
+        assert f.tracecount   == 414
+        assert list(f.ilines) == list(range(111, 111 + 23))
+        assert list(f.xlines) == list(range(875, 875 + 18))
+        assert f.dtype        == np.dtype(np.int16)
+
+def test_readtrace_int16():
+    with segyio.open('test-data/f3.sgy') as f:
+        tr = f.trace[10]
+        assert list(tr[20:45: 5]) == [0, -1170, 5198, -2213, -888]
+        assert list(tr[40:19:-5]) == [-888, -2213, 5198, -1170, 0]
+        assert list(tr[53:50:-1]) == [-2609, -2625, 681]
+
+
+@tmpfiles('test-data/f3.sgy')
+def test_writetrace_int16(tmpdir):
+    with segyio.open(tmpdir / 'f3.sgy', mode = 'r+') as f:
+        tr = np.asarray(range(len(f.samples)), dtype = np.int16)
+        f.trace[0] = tr
+        f.trace[1] = tr + 1
+
+    with segyio.open(tmpdir / 'f3.sgy') as f:
+        # read both with trace and raw-with-slice, since sliced raw allocates
+        # and internal buffer that must match the type
+        assert np.array_equal(f.trace[0], tr)
+        assert np.array_equal(f.trace[1], tr + 1)
+        assert np.array_equal(f.trace.raw[:2], [tr, tr+1])
+
+@tmpfiles('test-data/f3.sgy')
+def test_write_iline_int16(tmpdir):
+    with segyio.open(tmpdir / 'f3.sgy', mode = 'r+') as f:
+        shape = f.iline[f.ilines[0]].shape
+        il = np.arange(np.prod(shape), dtype = np.int16).reshape(shape)
+        f.iline[f.ilines[0]] = il
+
+    with segyio.open(tmpdir / 'f3.sgy') as f:
+        assert np.array_equal(f.iline[f.ilines[0]], il)
+
+
+@tmpfiles('test-data/small.sgy')
+def test_missing_format_ibmfloat_fallback(tmpdir):
+    with segyio.open(tmpdir / 'small.sgy', mode = 'r+') as f:
+        f.bin[segyio.su.format] = 0
+
+    with pytest.warns(UserWarning):
+        with segyio.open(tmpdir / 'small.sgy') as f:
+            assert int(f.format) == 1
+            assert f.dtype       == np.dtype(np.float32)
