@@ -115,7 +115,7 @@ def get_instance_segyiofd(tmpdir,
     path = str(tmpdir)
     f = os.path.join(path, file_name)
     if binary is not None:
-        return _segyio.segyiofd(f, mode, binary)
+        return _segyio.segyiofd(f, mode, tracecount, binary)
     else:
         return _segyio.segyiofd(f, mode)
 
@@ -159,7 +159,7 @@ def test_read_binary_header_fields(mmap=False):
     with pytest.raises(TypeError):
         _ = _segyio.getfield([], 0)
 
-    with pytest.raises(IndexError):
+    with pytest.raises(KeyError):
         _ = _segyio.getfield(binary_header, -1)
 
     assert _segyio.getfield(binary_header, 3225) == 1
@@ -359,10 +359,10 @@ def test_get_and_putfield():
     with pytest.raises(TypeError):
         _segyio.putfield({}, 0, 1)
 
-    with pytest.raises(IndexError):
+    with pytest.raises(KeyError):
         _segyio.getfield(hdr, 0)
 
-    with pytest.raises(IndexError):
+    with pytest.raises(KeyError):
         _segyio.putfield(hdr, 0, 1)
 
     _segyio.putfield(hdr, 1, 127)
